\name{OnDiskRaw-class}
\docType{class}

\alias{class:OnDiskRaw}
\alias{OnDiskRaw-class}
\alias{OnDiskRaw}

\alias{length,OnDiskRaw-method}


\title{OnDiskRaw objects}

\description{
  THIS IS A WORK-IN-PROGRESS!
}

\author{H. Pagès}

\keyword{methods}
\keyword{classes}
